"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var EBootState;
(function (EBootState) {
    EBootState["Initial"] = "initial";
    EBootState["Booting"] = "booting";
    EBootState["Finished"] = "finished";
})(EBootState = exports.EBootState || (exports.EBootState = {}));
class BootLoader {
    //private readonly logger: ILogger;
    constructor() {
        this.state = EBootState.Initial;
        this.bootFunctionObjects = [];
        this.afterBootFunctionObjects = [];
        //this.logger = loggerFactory.create(this.constructor.name);
    }
    getBootState() {
        return this.state;
    }
    isBooting() {
        return this.state === EBootState.Booting;
    }
    hasBooted() {
        return this.state === EBootState.Finished;
    }
    addBootFunction(name, fn) {
        //this.logger.trace("addBootFunction", name);
        this.bootFunctionObjects.push({ name, fn });
    }
    onBootReady(name, fn) {
        //this.logger.trace("onBootReady", name);
        if (this.state === EBootState.Finished) {
            return fn();
        }
        this.afterBootFunctionObjects.push({ name, fn });
    }
    getReadyPromise() {
        return new Promise((resolve, reject) => {
            this.onBootReady("BootLoader.ready", () => resolve());
        });
    }
    async boot() {
        this.state = EBootState.Booting;
        try {
            for (const fnObj of this.bootFunctionObjects) {
                //this.logger.trace("boot.bootFunctions.start", fnObj.name);
                await fnObj.fn(this);
                //this.logger.trace("boot.bootFunctions.end", fnObj.name);
            }
            this.state = EBootState.Finished;
            for (const fnObj of this.afterBootFunctionObjects) {
                //this.logger.trace("boot.afterBootFunctions.start", fnObj.name);
                fnObj.fn(this);
                //this.logger.trace("boot.afterBootFunctions.start", fnObj.name);
            }
        }
        catch (err) {
            //this.logger.error(`BootLoader.boot.error.caught: ${err}\n`);
            throw err;
        }
    }
}
exports.BootLoader = BootLoader;
//# sourceMappingURL=index.js.map