"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const _ = require("lodash");
const index_1 = require("../index");
const errors_1 = require("./errors");
const ConfigMergeHelper_1 = require("./helpers/ConfigMergeHelper");
const EnvironmentBuilder_1 = require("./helpers/EnvironmentBuilder");
class Config {
    constructor() {
        var _a, _b;
        this.applicationConfig = {};
        this.config = {};
        this.NODE_ENV = (_b = (_a = process.env) === null || _a === void 0 ? void 0 : _a.NODE_ENV) !== null && _b !== void 0 ? _b : "production";
        this.applicationConfig = this.loadApplicationConfig();
        this.registerConfig("Config", `${__dirname}/../config`);
        const namespace = this.config.Config.namespace;
        this.ENVIRONMENT = EnvironmentBuilder_1.default.buildEnvironment(this.NODE_ENV, namespace);
        index_1.Container.set("ENVIRONMENT", JSON.parse(JSON.stringify(this.ENVIRONMENT)));
    }
    loadApplicationConfig() {
        var _a, _b;
        const applicationConfigFolderPath = `${path.dirname((_b = (_a = require.main) === null || _a === void 0 ? void 0 : _a.filename) !== null && _b !== void 0 ? _b : "")}/config`;
        return this.getConfigFromConfigFiles(applicationConfigFolderPath);
    }
    getConfigFromConfigFiles(configDirectory) {
        const defaultConfigPath = `${configDirectory}/default.js`;
        const environmentConfigPath = `${configDirectory}/${this.NODE_ENV}.js`;
        let defaultConfig;
        try {
            defaultConfig = require(defaultConfigPath);
        }
        catch (err) {
            throw new errors_1.DefaultConfigNotFoundError(`config.default.loading.error.not.found: ${defaultConfigPath} \n ${err}`);
        }
        let environmentConfig;
        try {
            environmentConfig = require(environmentConfigPath);
        }
        catch (err) {
            environmentConfig = {};
        }
        return _.defaultsDeep(environmentConfig, defaultConfig);
    }
    applyConfigModule(name, baseConfigModule) {
        if (name in this.config)
            return;
        const applicationConfigOfModule = this.applicationConfig[name] || {};
        const processEnvironmentConfigOfModule = ConfigMergeHelper_1.default.getProcessEnvironmentConfig(name);
        const configModule = _.defaultsDeep(processEnvironmentConfigOfModule, applicationConfigOfModule, baseConfigModule);
        ConfigMergeHelper_1.default.checkForMissingConfigProperties(name, configModule);
        this.config[name] = configModule;
    }
    registerConfig(name, configDirectory) {
        const baseConfigModule = this.getConfigFromConfigFiles(configDirectory);
        this.applyConfigModule(name, baseConfigModule);
        return this.getConfig(name);
    }
    registerApplicationConfigModule(name) {
        const baseConfigModule = {};
        this.applyConfigModule(name, baseConfigModule);
        return this.getConfig(name);
    }
    getConfig(name) {
        if (!_.has(this.config, name)) {
            throw new Error(`config.module.not.found module name: ${name}`);
        }
        return _.cloneDeep(this.config[name]);
    }
    dangerouslyGetWholeConfig() {
        return _.cloneDeep(this.config);
    }
}
exports.Config = Config;
