"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const _ = require("lodash");
const ConfigMergeHelper_1 = require("./helpers/ConfigMergeHelper");
const EnvironmentBuilder_1 = require("./helpers/EnvironmentBuilder");
const Helper_1 = require("../Helper");
class ConfigManager {
    constructor() {
        var _a, _b, _c, _d, _e;
        this.applicationConfig = {};
        this.config = {};
        this.NODE_ENV = (_b = (_a = process.env) === null || _a === void 0 ? void 0 : _a.NODE_ENV) !== null && _b !== void 0 ? _b : "production";
        this.applicationConfig = this.loadApplicationConfig();
        this.registerConfig(this.constructor.name, `${__dirname}/../config`);
        const namespace = (_e = (_d = (_c = this.config) === null || _c === void 0 ? void 0 : _c.Config) === null || _d === void 0 ? void 0 : _d.namespace) !== null && _e !== void 0 ? _e : "one";
        this.ENVIRONMENT = EnvironmentBuilder_1.default.buildEnvironment(this.NODE_ENV, namespace);
    }
    loadApplicationConfig() {
        var _a, _b;
        const applicationConfigFolderPath = `${path.dirname((_b = (_a = require.main) === null || _a === void 0 ? void 0 : _a.filename) !== null && _b !== void 0 ? _b : "")}/config`;
        return this.getConfigFromConfigFiles(applicationConfigFolderPath);
    }
    getConfigFromConfigFiles(configDirectory) {
        const defaultConfigPath = `${configDirectory}/default.js`;
        const environmentConfigPath = `${configDirectory}/${this.NODE_ENV}.js`;
        let defaultConfig;
        try {
            defaultConfig = require(defaultConfigPath);
        }
        catch (err) {
            defaultConfig = {};
            const cleanUpDefaultConfigPath = Helper_1.Helper.cleanUpFilePath(defaultConfigPath);
            console.warn(`config.default.loading.error.not.found: Continuing without app configuration. Was expecting file at ${cleanUpDefaultConfigPath} ${defaultConfigPath}`);
        }
        let environmentConfig;
        try {
            environmentConfig = require(environmentConfigPath);
        }
        catch (err) {
            environmentConfig = {};
        }
        return _.defaultsDeep(environmentConfig, defaultConfig);
    }
    applyConfigModule(name, baseConfigModule) {
        if (name in this.config)
            return;
        const applicationConfigOfModule = this.applicationConfig[name] || {};
        const processEnvironmentConfigOfModule = ConfigMergeHelper_1.default.getProcessEnvironmentConfig(name);
        const configModule = _.defaultsDeep(processEnvironmentConfigOfModule, applicationConfigOfModule, baseConfigModule);
        ConfigMergeHelper_1.default.checkForMissingConfigProperties(name, configModule);
        this.config[name] = configModule;
    }
    registerConfig(name, configDirectory) {
        const baseConfigModule = this.getConfigFromConfigFiles(configDirectory);
        this.applyConfigModule(name, baseConfigModule);
        return this.getConfig(name);
    }
    registerApplicationConfigModule(name) {
        const baseConfigModule = {};
        this.applyConfigModule(name, baseConfigModule);
        return this.getConfig(name);
    }
    getConfig(name) {
        if (!_.has(this.config, name)) {
            throw new Error(`config.module.not.found module name: ${name}`);
        }
        return _.cloneDeep(this.config[name]);
    }
    dangerouslyGetWholeConfig() {
        return _.cloneDeep(this.config);
    }
}
exports.ConfigManager = ConfigManager;
//# sourceMappingURL=index.js.map